<?php
class HolidayComponent extends Object
{
const MONDAY = 2;
const TUESDAY = 3;
const WEDNESDAY = 4;

// ＰＨＰ版
// オリジナルはyy/mm/dd文字列ですが、シリアルで入れてください

// sample code
   // $aa = mktime(0,0,0,11,4,2002);
   // echo ktHolidayName($aa);
// --- end

//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
//_/
//_/  --- PHP 移植版 ( Update: 2020/11/28 ) ---
//_/
//_/  CopyRight(C) K.Tsunoda(AddinBox) 2001 All Rights Reserved.
//_/  ( AddinBox  http://addinbox.sakura.ne.jp/index.htm )
//_/  (  旧サイト  http://www.h3.dion.ne.jp/~sakatsu/index.htm )
//_/
//_/  この祝日判定コードは『Excel:kt関数アドイン』で使用している
//_/  ＶＢＡマクロを[PHP]に移植したものです。
//_/
//_/  この関数では以下の祝日変更までサポートしています。
//_/    (a) 2019年施行の「天皇誕生日の変更」 12/23⇒2/23 (補：2019年には[天皇誕生日]はありません)
//_/    (b) 2019年の徳仁親王の即位日(5/1) および
//_/       祝日に挟まれて「国民の休日」となる 4/30(平成天皇の退位日) ＆ 5/2 の２休日
//_/    (c) 2019年の「即位の礼 正殿の儀 (10/22) 」
//_/    (d) 2020年施行の「体育の日の改名」⇒スポーツの日
//_/    (e) 五輪特措法による2020年の「祝日移動」
//_/       海の日：7/20(3rd Mon)⇒7/23, スポーツの日:10/12(2nd Mon)⇒7/24, 山の日：8/11⇒8/10
//_/    (f) 五輪特措法による2021年の「祝日移動」
//_/       海の日：7/19(3rd Mon)⇒7/22, スポーツの日:10/11(2nd Mon)⇒7/23, 山の日：8/11⇒8/8
//_/
//_/  (*1)このコードを引用するに当たっては、必ずこのコメントも
//_/      一緒に引用する事とします。
//_/  (*2)他サイト上で本マクロを直接引用する事は、ご遠慮願います。
//_/      【 http://addinbox.sakura.ne.jp/holiday_logic.htm 】
//_/      へのリンクによる紹介で対応して下さい。
//_/  (*3)[ktHolidayName]という関数名そのものは、各自の環境に
//_/      おける命名規則に沿って変更しても構いません。
//_/ 
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

function ktHolidayName($MyDate)
{
  //define("MONDAY","2");   // 後述 Weekday関数 [日(1),月(2)‥‥土(7)]

  $cstImplementHolidayInLieu = mktime(0,0,0,4,12,1973);   // 振替休日施行

  $HolidayName = $this->prvHolidayChk($MyDate);

  // ----- 振替休日の判定 (振替休日施行日:1973/4/12) -----
  // [ 対象日≠祝日/休日 ＆ 対象日＝月曜日 ]のみ、前日(＝日曜日)を祝日判定する。
  // 前日(＝日曜日)が祝日の場合は”振替休日”となる。
  // 尚、５月６日の扱いを
  //     「火曜 or 水曜(みどりの日(5/4) or 憲法記念日(5/3)の振替休日)」⇒５月ブロック内で判定済
  //     「月曜(こどもの日(5/5)の振替休日」⇒ここの判定処理で判定
  // とする事により、ここでの判定対象は『対象日が月曜日』のみ となります。
  if ($HolidayName == "") {
      if ($this->Weekday($MyDate) == self::MONDAY) {
          if ($MyDate >= $cstImplementHolidayInLieu) {
              $YesterDay = mktime(0,0,0,$this->Month($MyDate),
                                      ($this->Day($MyDate) - 1),$this->Year($MyDate));
              $HolidayName = $this->prvHolidayChk($YesterDay);
              if ($HolidayName != "") {
                  $Result = "振替休日";
              } else {
                  $Result = "";
              }
          } else {
              $Result = "";
          }
      } else {
          $Result = "";
      }
  } else {
      $Result = $HolidayName;
  }

  return $Result;
}

//========================================================================

function prvHolidayChk($MyDate)
{
  //define("MONDAY","2");   // 後述 Weekday関数 [日(1),月(2)‥‥土(7)]
  //define("TUESDAY","3");
  //define("WEDNESDAY","4");

  $cstImplementTheLawOfHoliday = mktime(0,0,0,7,20,1948);  // 祝日法施行
  $cstShowaTaiso = mktime(0,0,0,2,24,1989);                // 昭和天皇大喪の礼
  $cstAkihitoKekkon = mktime(0,0,0,4,10,1959);             // 明仁親王の結婚の儀
  $cstNaruhitoKekkon = mktime(0,0,0,6,9,1993);             // 徳仁親王の結婚の儀
  $cstSokuireiSeiden = mktime(0,0,0,11,12,1990);           // 即位礼正殿の儀

  $cstTaii_Heisei = mktime(0,0,0,4,30,2019);               // 平成天皇の退位日, 祝日ではなく「国民の休日」です
  $cstSokui_Naruhito = mktime(0,0,0,5,1,2019);             // 即位の日(徳仁親王)
  $cst2019GW_May2nd = mktime(0,0,0,5,2,2019);              // 祝日ではなく「国民の休日」です
  $cstSokuireiSeiden_Naruhito = mktime(0,0,0,10,22,2019);  // 即位礼正殿の儀(徳仁親王)

  $MyYear = $this->Year($MyDate);
  $MyMonth = $this->Month($MyDate);
  $MyDay = $this->Day($MyDate);
  $MyDate = mktime(0,0,0,$MyMonth,$MyDay,$MyYear);

  if ($MyDate < $cstImplementTheLawOfHoliday)
      return "";    // 祝日法施行以前
  else;

  $Result = "";
  switch ($MyMonth) {
  // １月 //
  case 1:
      if ($MyDay == 1) {
          $Result = "元日";
      } else {
          if ($MyYear >= 2000) {
              $strNumberOfWeek =
                        (floor(($MyDay - 1) / 7) + 1) . $this->Weekday($MyDate);
              if ($strNumberOfWeek == "22") {    // 2nd Monday(2)
                  $Result = "成人の日";
              } else;
          } else {
              if ($MyDay == 15) {
                  $Result = "成人の日";
              } else;
          }
      }
      break;

  // ２月 //
  case 2:
      if ($MyDay == 11) {
          if ($MyYear >= 1967) {
              $Result = "建国記念の日";
          } else;
      } elseif ($MyDay == 23) {
          if ($MyYear >= 2020) {
              $Result = "天皇誕生日";
          } else;
      } elseif ($MyDate == $cstShowaTaiso) {
          $Result = "昭和天皇の大喪の礼";
      } else;
      break;

  // ３月 //
  case 3:
      if ($MyDay == $this->prvDayOfSpringEquinox($MyYear)) {    // 1948～2150以外は[99]
          $Result = "春分の日";                        // が返るので､必ず≠になる
      } else;
      break;

  // ４月 //
  case 4:
      if ($MyDay == 29) {
          if ($MyYear >= 2007) {
              $Result = "昭和の日";
          } elseif ($MyYear >= 1989) {
              $Result = "みどりの日";
          } else {
              $Result = "天皇誕生日";  // 昭和天皇
          }
      } elseif ($MyDate == $cstTaii_Heisei) {    // 2019/4/30(平成天皇の退位日)
          $Result = "国民の休日";    // 祝日に挟まれた国民の休日です
      } elseif ($MyDate == $cstAkihitoKekkon) {    // 1959/4/10
          $Result = "皇太子明仁親王の結婚の儀";
      } else;
      break;

  // ５月 //
  case 5:
      if ($MyDay == 3) {
          $Result = "憲法記念日";
      } elseif ($MyDay == 4) {
          if ($MyYear >= 2007) {
              $Result = "みどりの日";
          } elseif ($MyYear >= 1986) {
              if ($this->Weekday($MyDate) > self::MONDAY) {  // 火曜 以降(火～土)
                  // 5/4が日曜日は『只の日曜』､月曜日は『憲法記念日の振替休日』(～2006年)
                  $Result = "国民の休日";
              } else;
          } else;
      } elseif ($MyDay == 5) {
          $Result = "こどもの日";
      } elseif ($MyDay == 6) {
          if ($MyYear >= 2007) {
              if (($this->Weekday($MyDate) == self::TUESDAY) || ($this->Weekday($MyDate) == self::WEDNESDAY)) {
                  $Result = "振替休日";    // [5/3,5/4が日曜]ケースのみ、ここで判定
              } else;
          } else;
      } else {
          if ($MyYear == 2019) {
              if ($MyDate == $cstSokui_Naruhito) {    // 2019/5/1
                  $Result = "即位の日";    // 徳仁親王
              } elseif ($MyDate == $cst2019GW_May2nd) {    // 2019/5/2
                  $Result = "国民の休日";    //祝日に挟まれた国民の休日です
              } else;
          } else;
      }
      break;

  // ６月 //
  case 6:
      if ($MyDate == $cstNaruhitoKekkon) {
          $Result = "皇太子徳仁親王の結婚の儀";
      } else;
      break;

  // ７月 //
  case 7:
      $strNumberOfWeek = (floor(($MyDay - 1) / 7) + 1) . $this->Weekday($MyDate);
      if ($MyYear >= 2022) {
          if ($strNumberOfWeek == "32") {    // 3rd Monday(2)
              $Result = "海の日";
          } else;
      } elseif ($MyYear == 2021) {
          // 2021年はオリンピック特措法により
          //「海の日」が 7/22 , 「スポーツの日」が 7/23 に移動
          if ($MyDay == 22) {
              $Result = "海の日";
          } elseif ($MyDay == 23) {
              $Result = "スポーツの日";
          } else;
      } elseif ($MyYear == 2020) {
          // 2020年はオリンピック特措法により
          //「海の日」が 7/23 , 「スポーツの日」が 7/24 に移動
          if ($MyDay == 23) {
              $Result = "海の日";
          } elseif ($MyDay == 24) {
              $Result = "スポーツの日";
          } else;
      } elseif ($MyYear >= 2003) {
          if ($strNumberOfWeek == "32") {    // 3rd Monday(2)
              $Result = "海の日";
          } else;
      } elseif ($MyYear >= 1996) {
          if ($MyDay == 20) {
              $Result = "海の日";
          } else;
      } else;
      break;

  // ８月 //
  case 8:
      if ($MyYear >= 2022) {
          if ($MyDay == 11) {
              $Result = "山の日";
          } else;
      } elseif ($MyYear == 2021) {
          // 2021年はオリンピック特措法により「山の日」が 8/8 に移動
          if ($MyDay == 8) {
              $Result = "山の日";
          } else;
      } elseif ($MyYear == 2020) {
          // 2020年はオリンピック特措法により「山の日」が 8/10 に移動
          if ($MyDay == 10) {
              $Result = "山の日";
          } else;
      } elseif ($MyYear >= 2016) {
          if ($MyDay == 11) {
              $Result = "山の日";
          } else;
      } else;
      break;

  // ９月 //
  case 9:
      //第３月曜日(15～21)と秋分日(22～24)が重なる事はない
      $MyAutumnEquinox = $this->prvDayOfAutumnEquinox($MyYear);
      if ($MyDay == $MyAutumnEquinox) {    // 1948～2150以外は[99]
          $Result = "秋分の日";            // が返るので､必ず≠になる
      } else {
          if ($MyYear >= 2003) {
              $strNumberOfWeek =
                      (floor(($MyDay - 1) / 7) + 1) . $this->Weekday($MyDate);
              if ($strNumberOfWeek == "32") {    // 3rd Monday(2)
                  $Result = "敬老の日";
              } elseif ($this->Weekday($MyDate) == self::TUESDAY) {
                  if ($MyDay == ($MyAutumnEquinox - 1)) {
                      $Result = "国民の休日";    // 火曜日＆[秋分日の前日]
                  } else;
              } else;
          } elseif ($MyYear >= 1966) {
              if ($MyDay == 15) {
                  $Result = "敬老の日";
              } else;
          } else;
      }
      break;

  // １０月 //
  case 10:
      $strNumberOfWeek = (floor(( $MyDay - 1) / 7) + 1) . $this->Weekday($MyDate);
      if ($MyYear >= 2022) {
          if ($strNumberOfWeek == "22") {    // 2nd Monday(2)
              $Result = "スポーツの日";      // 2020年より改名
          } else;
      } elseif (($MyYear == 2020) || ($MyYear == 2021)) {
          // 2021年はオリンピック特措法により「スポーツの日」が 7/23 に移動
          // 2020年はオリンピック特措法により「スポーツの日」が 7/24 に移動
      } elseif ($MyYear >= 2000) {
          if ($strNumberOfWeek == "22") {    // 2nd Monday(2)
              $Result = "体育の日";
          } elseif ($MyDate == $cstSokuireiSeiden_Naruhito) {    // 2019/10/22
              $Result = "即位礼正殿の儀";    // 徳仁親王
          } else;
      } elseif ($MyYear >= 1966) {
          if ($MyDay == 10) {
              $Result = "体育の日";
          } else;
      } else;
      break;

  // １１月 //
  case 11:
      if ($MyDay == 3) {
          $Result = "文化の日";
      } elseif ($MyDay == 23) {
          $Result = "勤労感謝の日";
      } elseif ($MyDate == $cstSokuireiSeiden) {
          $Result = "即位礼正殿の儀";  // 平成天皇
      } else;
      break;

  // １２月 //
  case 12:
      if ($MyDay == 23) {
          if (($MyYear >= 1989) && ($MyYear <= 2018)) {
              $Result = "天皇誕生日";  // 平成天皇
          } else;
      } else;
      break;
  }
  return $Result;
}

//======================================================================
//  春分/秋分日の略算式は
//    『海上保安庁水路部 暦計算研究会編 新こよみ便利帳』
//  で紹介されている式です。
function prvDayOfSpringEquinox($MyYear)
{
  if ($MyYear <= 1947)
      $Result = 99; //祝日法施行前
  elseif ($MyYear <= 1979)
      // floor 関数は[VBAのInt関数]に相当
      $Result = floor(20.8357 +
            (0.242194 * ($MyYear - 1980)) - floor(($MyYear - 1980) / 4));
  elseif ($MyYear <= 2099)
      $Result = floor(20.8431 +
            (0.242194 * ($MyYear - 1980)) - floor(($MyYear - 1980) / 4));
  elseif ($MyYear <= 2150)
      $Result = floor(21.851 +
            (0.242194 * ($MyYear - 1980)) - floor(($MyYear - 1980) / 4));
  else
      $Result = 99; //2151年以降は略算式が無いので不明

  return $Result;
}

//========================================================================
function prvDayOfAutumnEquinox($MyYear)
{
  if ($MyYear <= 1947)
      $Result = 99; //祝日法施行前
  elseif ($MyYear <= 1979)
      // floor 関数は[VBAのInt関数]に相当
      $Result = floor(23.2588 +
            (0.242194 * ($MyYear - 1980)) - floor(($MyYear - 1980) / 4));
  elseif ($MyYear <= 2099)
      $Result = floor(23.2488 +
            (0.242194 * ($MyYear - 1980)) - floor(($MyYear - 1980) / 4));
  elseif ($MyYear <= 2150)
      $Result = floor(24.2488 +
            (0.242194 * ($MyYear - 1980)) - floor(($MyYear - 1980) / 4));
  else
      $Result = 99; //2151年以降は略算式が無いので不明

  return $Result;
}

//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
//_/  CopyRight(C) K.Tsunoda(AddinBox) 2001 All Rights Reserved.
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

//---- ktHolidayName用  日付情報サポート関数 ----
function Weekday($MyDate){
  return strftime("%w",$MyDate) + 1;  // 日(0)～土(6) に１加算して[日(1),月(2)‥‥土(7)]で返す
}

function Year($MyDate){
  return strftime("%Y",$MyDate) - 0;  // 数値で返す(4桁 西暦年 [cf. 2018] )
}

function Month($MyDate){
  return strftime("%m",$MyDate) - 0;  // 数値で返す(1～12)
}

function Day($MyDate){
  return strftime("%d",$MyDate) - 0;  // 数値で返す(1～31)
}

//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
//_/  Visual Basic Compatibility functions for PHP
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
}
?>